if Network:is_client() then
	return
end

function GroupAIStateBesiege:get_all_needed_doors()

	local nav_manager = managers.navigation
	local all_doors = nav_manager._room_doors
	local all_nav_segs = nav_manager._nav_segments

	local reinforce_doors = {}
	for nav_seg_id, nav_seg_data in pairs(all_nav_segs) do
		if not managers.groupai:state():is_nav_seg_safe(nav_seg_id) then
			for neighbour_seg_id, door_list in pairs(nav_seg_data.neighbours) do
				if not all_nav_segs[neighbour_seg_id].disabled then
					for _, i_door in ipairs(door_list) do
						if type(i_door) == "number" then
							reinforce_doors[i_door] = all_doors[i_door]
						end
					end
				end
			end
		end
	end

	return reinforce_doors
end

function GroupAIStateBesiege:_find_spawn_points_near_pos(pos)
	local target_area = self:get_area_from_nav_seg_id(managers.navigation:get_nav_seg_from_pos(pos))
	local to_search_areas = {
		target_area
	}
	local found_areas = {
		[target_area] = true
	}

	local points = {}
	repeat
		local search_area = table.remove(to_search_areas, 1)

		for _, spawn_group in pairs(search_area.spawn_groups or {}) do
			if spawn_group.spawn_pts then
				for _, sp_data in pairs(spawn_group.spawn_pts) do
					if sp_data.accessibility ~= "acrobatic" then
						table.insert(points, sp_data)
					end
				end
			end
		end

		if #points > 0 then
			return points
		end

		for _, other_area in pairs(search_area.neighbours) do
			if not found_areas[other_area] then
				table.insert(to_search_areas, other_area)
				found_areas[other_area] = true
			end
		end
	until #to_search_areas == 0
end

Hooks:PostHook(GroupAIStateBesiege, "_upd_police_activity", "_upd_police_activity_smart_shields", function (self)
	if not self._enemy_weapons_hot then
		return
	end

	local needed_doors = self:get_all_needed_doors()

	local free_units = {}
	for u_key, _ in pairs(self._special_units.shield or {}) do
		local u_data = self._police[u_key]
		if u_data and alive(u_data.unit) and not needed_doors[u_data.door_id] then
			table.insert(free_units, u_data.unit)
			if u_data.door_data then
				u_data.door_data.assigned_shield = nil
				u_data.unit:brain():set_objective(nil)
			end
		end
	end

	local shield_category = tweak_data.group_ai.unit_categories.FBI_shield or tweak_data.group_ai.unit_categories.CS_shield
	local shield_unit = shield_category.unit_types[tweak_data.levels:get_ai_group_type()][1]

	local spawns = 3
	for door_id, door_data in pairs(needed_doors) do
		if not alive(door_data.assigned_shield) then
			local unit = table.remove(free_units)
			if alive(unit) then
				if unit:brain()._logic_data.group then
					self:unit_leave_group(unit)
				end
			else
				local spawn_points = self:_find_spawn_points_near_pos(door_data.center)
				if spawn_points then
					unit = table.random(spawn_points).mission_element:produce({name = shield_unit})
					if unit then
						self:set_char_team(unit, "law1")
						spawns = spawns - 1
					end
				end
			end

			if alive(unit) then
				unit:brain():set_objective({
					scan = true,
					type = "sniper",
					pos = door_data.center,
					nav_seg = managers.navigation:get_nav_seg_from_pos(door_data.center),
				})

				local u_data = self._police[unit:key()]
				door_data.assigned_shield = unit

				u_data.door_data = door_data
				u_data.door_id = door_id
			else
				break
			end

			if spawns <= 0 then
				break
			end
		end
	end

end)

Hooks:PreHook(GroupAIStateBesiege, "on_enemy_unregistered", "on_enemy_unregistered_smart_shields", function (self, unit)
	local u_key = unit:key()
	local u_data = self._police[u_key]
	if not u_data or not u_data.door_data then
		return
	end

	u_data.door_data.assigned_shield = nil
end)
